@@ -28,9 +28,13 @@
 #ifdef HAVE_STDLIB_H
   #include <stdlib.h>
 #endif
+#include <unistd.h>
 #include <math.h>
 #include <time.h>
 #include <string.h>
+#include <ctype.h>
+#include <glob.h>
+#include <fcntl.h>
 
 /* GTK */
 #include <gtk/gtk.h>
@@ -46,6 +50,7 @@
 /* graphical symbols */
 #include "in.xpm"
 #include "out.xpm"
+#include "record.xpm"
 
 /*
  * returns a new data structure (for association with a row)
@@ -82,6 +87,34 @@ void cid_toggle_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   }
 }
 
+/*
+ * Callback: called when a button was clicked in row delete dialog
+ */
+static void cid_delete_response_cb(GtkWidget* widget, gint response_id,
+                                   gpointer data)
+{
+  session_t* session = (session_t*) data;
+  guint row = (guint) g_object_get_data(G_OBJECT(widget), "row");
+  char* filename = (char*) g_object_get_data(G_OBJECT(widget), "filename");
+  GtkWidget* checkbutton =
+    (GtkWidget*) g_object_get_data(G_OBJECT(widget), "checkbutton");
+
+  if (response_id == GTK_RESPONSE_OK) {
+    if (filename &&
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton)))
+    {
+      unlink(filename);
+    }
+    cid_mark_row(session, row, FALSE); /* to count unanswered calls */
+    session->cid_num--;
+    /* decrease before removal: "select-row" signal submission -> other cb */
+    gtk_clist_remove(GTK_CLIST(session->cid_list), row);
+  }
+  
+  if (filename) free(filename);
+  gtk_widget_destroy(widget);
+}
+
 /*
  * called on cid delete request
  *
@@ -89,12 +122,44 @@ void cid_toggle_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
  */
 static void cid_request_delete(GtkWidget *widget _U_, gpointer data,
                                guint row) {
-  session_t *session = (session_t *) data;
+  session_t* session = (session_t *) data;
+  GtkWidget* vbox;
+  GtkWidget* label;
+  GtkWidget* checkbutton;
+  char *filename = cid_get_record_filename(session, row);
   
-  cid_mark_row(session, row, FALSE);
-  session->cid_num--;
-  /* decrease before removal: "select-row" signal submission -> other cb */
-  gtk_clist_remove(GTK_CLIST(session->cid_list), row);
+  GtkWidget* dialog = gtk_dialog_new_with_buttons(_("Delete Entry"),
+                         GTK_WINDOW(session->main_window),
+			 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+		         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK, GTK_RESPONSE_OK,
+			 NULL);
+
+  vbox = gtk_vbox_new(FALSE, 16);
+  gtk_container_set_border_width(GTK_CONTAINER(vbox), 16);
+  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox);
+  gtk_widget_show(vbox);
+  
+  label = gtk_label_new(_("Are you sure you want to\ndelete this entry?"));
+  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), label);
+  gtk_widget_show(label);
+
+  checkbutton = gtk_check_button_new_with_label(_("Delete recording"));
+  if (filename) { /* default: checked */
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+  } else { /* if file doesn't exist, grey out */
+    gtk_widget_set_sensitive(checkbutton, FALSE);
+  }
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), checkbutton);
+  gtk_widget_show(checkbutton);
+
+  g_object_set_data(G_OBJECT(dialog), "row", (gpointer) row);
+  g_object_set_data(G_OBJECT(dialog), "filename", (gpointer) filename);
+  g_object_set_data(G_OBJECT(dialog), "checkbutton", (gpointer) checkbutton);
+  g_signal_connect(dialog, "response", G_CALLBACK(cid_delete_response_cb),
+                   session);
+  gtk_widget_show(dialog);
 }
 
 /*
@@ -116,6 +181,142 @@ static gint cid_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data) {
   }
 }
 
+/*
+ * Callback: called on playback request
+ */
+static void cid_playback(GtkWidget *widget _U_, gpointer data, guint row) {
+  session_t *session = (session_t *) data;
+
+  session->effect_filename = cid_get_record_filename(session, row);
+  session->effect_playback_start_time = time(NULL);
+  session_set_state(session, STATE_PLAYBACK);
+}
+
+/*
+ * Callback: called on "OK" click in "Save as..." file selection
+ */
+static void cid_save_as_filename_cb(GtkWidget *fs) {
+  char* sourcename = g_object_get_data(G_OBJECT(fs), "sourcename");
+  const char* destbase =
+    gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+  if (*destbase && *basename(destbase)) {
+    /* copy if something was entered */
+    char* destination;
+    char* extension = filename_extension(sourcename);
+    if (0 <= asprintf(&destination, "%s.%s", destbase, extension)) {
+      /* actually copy if source and destination known */
+      int buffer_size = 65536;
+      char* buffer = (char*) malloc(buffer_size);
+      if (buffer) {
+	int fd_in = open(sourcename, O_RDONLY);
+	if (fd_in != -1) {
+	  int fd_out = open(destination, O_WRONLY | O_CREAT | O_TRUNC,
+	                    S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
+	  if (fd_out != -1) {
+	    int just_read;
+	    do {
+	      int to_write = just_read = read(fd_in, buffer, buffer_size);
+	      int written = 1;
+	      char* buffer_index = buffer;
+	      while (to_write && written) {
+		written = write(fd_out, buffer_index, to_write);
+		to_write -= written;
+		buffer_index += written;
+	      }
+	    } while (just_read);
+	    if (close(fd_out)) {
+	      fprintf(stderr, "Error on closing destination file.\n");
+	    }
+	  } else {
+	    fprintf(stderr,
+		    "Error on destination file (%s) open().\n", destination);
+	  }
+	  if (close(fd_in)) {
+	    fprintf(stderr, "Error on source file.\n");
+	  }
+	} else {
+	  fprintf(stderr,
+	          "Error on source file (%s) open().\n", sourcename);
+	}
+	free(buffer);
+      } else {
+	fprintf(stderr, "Error on malloc().\n");
+      }
+      free(destination);
+    } else {
+      fprintf(stderr, "Error on asprintf().\n");
+    }
+  }
+  free(sourcename);
+  gtk_widget_destroy(GTK_WIDGET(fs));
+}
+
+/*
+ * Callback: called on "Save as..." request
+ */
+static void cid_save_as(GtkWidget *widget _U_, gpointer data, guint row) {
+  session_t* session = (session_t*) data;
+  char* sourcename = cid_get_record_filename(session, row);
+  char* extension = filename_extension(sourcename);
+
+  if (extension) {
+    GtkWidget* fs;
+    char* title;
+    
+    if (0 > asprintf(&title, _("Enter the base filename for %s file"),
+	             extension))
+    {
+      fprintf(stderr, "Error on asprintf().\n");
+      return;
+    }
+    fs = gtk_file_selection_new(title);
+    free(title);
+    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
+                             "clicked", G_CALLBACK(gtk_widget_destroy),
+			     (gpointer) fs);
+    g_object_set_data(G_OBJECT(fs), "sourcename", sourcename);
+    g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button),
+                           "clicked", G_CALLBACK(cid_save_as_filename_cb), fs);
+    gtk_widget_show(fs);
+  } else {
+    fprintf(stderr, "Error: no filename extension found.\n");
+  }
+}
+
+/*
+ * Callback: called if something happened in the delete recording dialog
+ */
+static void cid_delete_rec_response_cb(GtkWidget* widget, gint response_id,
+                                       gpointer data)
+{
+  session_t *session = (session_t *) data;
+
+  if (response_id == GTK_RESPONSE_OK) {
+    char* temp;
+    guint row = (guint) g_object_get_data(G_OBJECT(widget), "row");
+    unlink(temp = cid_get_record_filename(session, row));
+    free(temp);
+    cid_row_mark_record(session, row);
+  }
+  gtk_widget_destroy(widget);
+}
+
+/*
+ * Callback: called on delete recording request
+ */
+static void cid_delete_rec(GtkWidget *widget _U_, gpointer data, guint row) {
+  session_t *session = (session_t *) data;
+
+  GtkWidget* dialog = gtk_message_dialog_new(GTK_WINDOW(session->main_window),
+      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING,
+      GTK_BUTTONS_OK_CANCEL, _("Do you really want to delete this recording?"));
+  g_signal_connect(dialog, "response", G_CALLBACK(cid_delete_rec_response_cb),
+                   session);
+  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+  g_object_set_data(G_OBJECT(dialog), "row", (gpointer) row);
+  gtk_widget_show(dialog);
+}
+
 /*
  * called on mouse button pressed in cid list
  */
@@ -132,8 +333,9 @@ static gint cid_mouse_cb(GtkWidget *widget _U_,
 
       GtkItemFactoryEntry menu_items[] = {
 /*path                   accel. callb.         cb param. kind           extra */
-{_("/_Playback"),        NULL,  NULL,          0,        "",            NULL},
-{_("/Delete _Recording"),NULL,  NULL,          0,        "",            NULL},
+{_("/_Playback"),        NULL,  cid_playback,  row,      "",            NULL},
+{_("/_Save as..."),      NULL,  cid_save_as,   row,      "",            NULL},
+{_("/Delete _Recording"),NULL,  cid_delete_rec,row,      "",            NULL},
 {  "/Sep",               NULL,  NULL,          0,        "<Separator>", NULL},
 {_("/_Delete Row"),      NULL,  cid_request_delete, row, "",            NULL}
       };
@@ -144,21 +346,43 @@ static gint cid_mouse_cb(GtkWidget *widget _U_,
       gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
 
       GtkWidget *playback_item;
+      GtkWidget *save_as_item;
       GtkWidget *delete_record_item;
       GtkWidget *delete_item;
+      char* fn;
+      char* temp;
   
       accel_group = gtk_accel_group_new();
       item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<popup>",
-				      accel_group);
+	                                  accel_group);
       gtk_item_factory_create_items_ac(item_factory, nmenu_items, menu_items,
 				   session, 2);
-      playback_item = gtk_item_factory_get_item(item_factory, _("/Playback"));
-      delete_record_item =
-	gtk_item_factory_get_item(item_factory, _("/Delete Recording"));
-      delete_item = gtk_item_factory_get_item(item_factory, _("/Delete Row"));
- 
-      gtk_widget_set_sensitive(playback_item, FALSE);
-      gtk_widget_set_sensitive(delete_record_item, FALSE);
+      
+      if (!(playback_item = gtk_item_factory_get_item(item_factory,
+	  temp = stripchr(_("/_Playback"), '_'))))
+        fprintf(stderr, "Error getting playback_item.\n");
+      free(temp);
+      if (!(save_as_item = gtk_item_factory_get_item(item_factory,
+	  temp = stripchr(_("/_Save as..."), '_'))))
+        fprintf(stderr, "Error getting save_as_item.\n");
+      free(temp);
+      if (!(delete_record_item = gtk_item_factory_get_item(item_factory,
+	    temp = stripchr(_("/Delete _Recording"), '_'))))
+        fprintf(stderr, "Error getting delete_record_item_item.\n");
+      free(temp);
+      if (!(delete_item = gtk_item_factory_get_item(item_factory,
+	  temp = stripchr(_("/_Delete Row"), '_'))))
+        fprintf(stderr, "Error getting delete_item.\n");
+      free(temp);
+
+      if (!(session->state == STATE_READY &&
+	    (fn = cid_get_record_filename(session, row)))) {
+        gtk_widget_set_sensitive(playback_item, FALSE);
+        gtk_widget_set_sensitive(save_as_item, FALSE);
+        gtk_widget_set_sensitive(delete_record_item, FALSE);
+      } else {
+	free(fn);
+      }
   
       menu = GTK_MENU(gtk_item_factory_get_widget(item_factory, "<popup>"));
       gtk_menu_set_accel_group(menu, accel_group);
@@ -192,7 +416,9 @@ GtkWidget *cid_new(session_t *session) {
   GtkWidget *scrolled_window; /* the home for the clist */
   GtkWidget *clist;           /* the list itself */
   GtkStyle *style;            /* needed for preparing symbols */
+  GtkWidget *pixmapwidget;    /* record symbol */
 
+  style = gtk_widget_get_style(session->main_window);
   gchar *titles[CID_COL_NUMBER];
   frame = gtk_frame_new(_("Caller ID"));
 
@@ -216,6 +442,14 @@ GtkWidget *cid_new(session_t *session) {
 
   /* the list widget */
   clist = gtk_clist_new_with_titles(CID_COL_NUMBER, titles);
+  session->symbol_record_pixmap = gdk_pixmap_create_from_xpm_d(
+					 session->main_window->window,
+					 &session->symbol_record_bitmap,
+					 &style->bg[GTK_STATE_NORMAL],
+					 (gchar **) record_xpm);
+  pixmapwidget = gtk_pixmap_new(session->symbol_record_pixmap,
+                                session->symbol_record_bitmap);
+  gtk_clist_set_column_widget(GTK_CLIST(clist), CID_COL_FLAGS, pixmapwidget);
   
   /* default: GTK_SELECTION_MULTIPLE: */
   gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
@@ -242,7 +476,6 @@ GtkWidget *cid_new(session_t *session) {
   gtk_widget_show(clist);
 
   /* set up IN/OUT symbols */
-  style = gtk_widget_get_style(session->main_window);
   session->symbol_in_pixmap = gdk_pixmap_create_from_xpm_d(
 				 session->main_window->window,
 				 &session->symbol_in_bitmap,
@@ -351,7 +584,7 @@ void cid_add_line(session_t *session,
                         CID_COL_TYPE, typestring, 50, pixmap, bitmap);
 
   cid_jump_to_end(session);
-  
+
   /* clean up */
   free(line[CID_COL_TIME]);
 }
@@ -412,6 +645,9 @@ void cid_add_saved_line(session_t *session, char *date, char *type,
 		     session->cid_num - 1, CID_COL_TIME, date);
   gtk_clist_set_text(GTK_CLIST(session->cid_list),
 		     session->cid_num - 1, CID_COL_DURATION, duration);
+
+  cid_row_mark_record(session, session->cid_num - 1);
+  
 }
 
 /*
@@ -588,3 +824,93 @@ void cid_mark_row(session_t *session, int row, int state) {
   }
 }
 
+/*
+ * returns timestring with just digits
+ * - caller has got to free() the result
+ */
+char* cid_purify_timestring(char *s) {
+  char* t;
+  int length = 0;
+  char* result;
+  char* result_index;
+
+  for (t=s; *t; t++) {
+    if (isdigit (*t))
+      length++;
+  }
+  
+  result_index = result = (char*) malloc (length + 1);
+  
+  for (t=s; *t; t++) {
+    if (isdigit (*t)) {
+      *(result_index++) = *t;
+    }
+  }
+  *(result_index++) = '\0';
+ 
+  return result;
+}
+
+/*
+ * returns name of sound filename, if it exists; NULL otherwise
+ * - caller has go to free() result
+ */
+char* cid_get_record_filename(session_t* session, int row) {
+  char* timestr;
+  char* homedir;
+  char* pattern;
+  glob_t g;
+  char* result; /* found something */
+  
+  gtk_clist_get_text(GTK_CLIST(session->cid_list), row, CID_COL_TIME, &timestr);
+  timestr = cid_purify_timestring(timestr);
+
+  if (!(homedir = get_homedir())) {
+    fprintf(stderr, "Warning: Couldn't get home dir.\n");
+    return NULL;
+  }
+  
+  if (asprintf(&pattern, "%s/." PACKAGE "/recordings/%s.*",
+	homedir, timestr) < 0) {
+    fprintf(stderr, "Warning: "
+	    "Couldn't allocate memory for filename globbing pattern.\n");
+    return NULL;
+  }
+  
+  switch (glob(pattern, 0, NULL, &g)) {
+    case 0:
+      result = strdup(*(g.gl_pathv));
+      break;
+    case GLOB_NOMATCH:
+      result = NULL;
+      break;
+    default:
+      fprintf(stderr, "Warning: "
+	  "globbing error while looking up recorded conversation.\n");
+      return NULL;
+  }
+
+  globfree(&g);
+  free(pattern);
+  free(timestr);
+
+  return result;
+}
+
+/*
+ * marks row with record sign if recording available
+ * - deletes mark if recording was removed before
+ */
+void cid_row_mark_record(session_t* session, int row) {
+  char* fn;
+  
+  if ((fn = cid_get_record_filename(session, row))) {
+    gtk_clist_set_pixmap(GTK_CLIST(session->cid_list), row, CID_COL_FLAGS,
+        session->symbol_record_pixmap, session->symbol_record_bitmap);
+    free(fn);
+  } else {
+    gtk_clist_set_text(GTK_CLIST(session->cid_list), row, CID_COL_FLAGS, "");
+  }
+  
+}
+
