@@ -124,17 +124,36 @@ gint timeout_callback(gpointer data) {
 
   if (session->state == STATE_CONVERSATION) {
     char *timediff = timediff_str(time(NULL), session->vcon_time);
-    char buf[64];
+    char *buf;
 
-    if (snprintf(buf, 64, "%s %s",
-		 state_data[STATE_CONVERSATION].status_bar, timediff) >= 64)
-      buf[63] = '\0';
+    if (0 > asprintf(&buf, "%s %s",
+		     state_data[session->state].status_bar, timediff))
+      fprintf(stderr, "Warning: timeout_callback: asprintf error.\n");
     
     gtk_statusbar_pop(GTK_STATUSBAR(session->status_bar),
 		      session->phone_context_id);
     gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
 		       session->phone_context_id,
 		       buf);
+    free(buf);
+    free(timediff);
+  }
+  
+  if (session->state == STATE_PLAYBACK) {
+    char *timediff = timediff_str(time(NULL),
+	                          session->effect_playback_start_time);
+    char *buf;
+
+    if (0 > asprintf(&buf, "%s %s",
+		     state_data[session->state].status_bar, timediff))
+      fprintf(stderr, "Warning: timeout_callback: asprintf error.\n");
+    
+    gtk_statusbar_pop(GTK_STATUSBAR(session->status_bar),
+		      session->phone_context_id);
+    gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
+		       session->phone_context_id,
+		       buf);
+    free(buf);
     free(timediff);
   }
   
@@ -210,9 +229,9 @@ void show_audio_error_dialog() {
   gtk_widget_show(dialog);
 }
 
-/*
- * some GUI callbacks
- */
+/**********************
+ * some GUI callbacks *
+ **********************/
 
 /*
  * File Menu entries
@@ -449,6 +468,7 @@ GtkWidget *get_main_menu(GtkWidget *window, session_t *session) {
 
   GtkItemFactory *item_factory;
   GtkAccelGroup *accel_group;
+  char* temp;
 
   gint nmenu_items = sizeof(main_menu_items) / sizeof(main_menu_items[0]);
 
@@ -461,15 +481,20 @@ GtkWidget *get_main_menu(GtkWidget *window, session_t *session) {
   
   /* save connection to menu item(s), set defaults */
   session->cid_check_menu_item = gtk_item_factory_get_item(item_factory,
-				   _("/View/Caller ID Monitor"));
+		   temp = stripchr(_("/View/_Caller ID Monitor"), '_'));
+  free(temp);
   session->llcheck_check_menu_item = gtk_item_factory_get_item(item_factory,
-				   _("/View/Line Level Meters"));
+		   temp = stripchr(_("/View/_Line Level Meters"), '_'));
+  free(temp);
   session->controlpad_check_menu_item = gtk_item_factory_get_item(item_factory,
-				   _("/View/Control Pad"));
+		   temp = stripchr(_("/View/Control _Pad"), '_'));
+  free(temp);
   session->menuitem_settings = gtk_item_factory_get_item(item_factory,
-				   _("/Options/Settings"));
+		   temp = stripchr(_("/Options/_Settings"), '_'));
+  free(temp);
   session->menuitem_line_check = gtk_item_factory_get_item(item_factory,
-				   _("/Phone/Line Level Check"));
+		   temp = stripchr(_("/Phone/_Line Level Check"), '_'));
+  free(temp);
   gtk_check_menu_item_set_active(
     GTK_CHECK_MENU_ITEM(session->cid_check_menu_item),
     session->option_show_callerid);
