@@ -48,5 +48,7 @@ int touch_dir(char *dirname);
 int touch_dotdir();
 int substitute(char **s, char *x, char *y);
 char *util_digitstime(time_t *time);
+char* stripchr(char* s, char c);
+char* filename_extension(char* fn);
 
 #endif /* util.h */
